@@ -22,7 +22,7 @@
 
 """Interpolation of geographical tiepoints using the second order interpolation
 scheme implemented in the CVIIRS software, as described here:
-Compact VIIRS SDR Product Format User Guide (V1J) 
+Compact VIIRS SDR Product Format User Guide (V1J)
 http://www.eumetsat.int/website/wcm/idc/idcplg?IdcService=GET_FILE&dDocName=PDF_DMT_708025&RevisionSelectionMethod=LatestReleased&Rendition=Web
 """
 
@@ -30,9 +30,6 @@ import xarray as xr
 import dask.array as da
 import numpy as np
 
-# TODO on interpolation:
-# - go over to cartesian coordinates for tricky situation (eg poles, dateline)
-
 R = 6371.
 # Aqua scan width and altitude in km
 scan_width = 10.00017
@@ -86,7 +83,7 @@ def get_corners(arr):
 
 class ModisInterpolator():
 
-    def __init__(self, cres, fres):
+    def __init__(self, cres, fres, cscan_full_width=None):
         if cres == 1000:
             self.cscan_len = 10
             self.cscan_width = 1
@@ -94,7 +91,10 @@ class ModisInterpolator():
         elif cres == 5000:
             self.cscan_len = 2
             self.cscan_width = 5
-            self.cscan_full_width = 271
+            if cscan_full_width is None:
+                self.cscan_full_width = 271
+            else:
+                self.cscan_full_width = cscan_full_width
 
         if fres == 250:
             self.fscan_width = 4 * self.cscan_width
@@ -135,7 +135,10 @@ class ModisInterpolator():
     def _expand_tiepoint_array_5km(self, arr, lines, cols):
         arr = da.repeat(arr, lines * 2, axis=1)
         arr = da.repeat(arr.reshape((-1, self.cscan_full_width - 1)), cols, axis=1)
-        return da.hstack((arr[:, :2], arr, arr[:, -2:]))
+        if self.cscan_full_width == 271:
+            return da.hstack((arr[:, :2], arr, arr[:, -2:]))
+        else:
+            return da.hstack((arr[:, :2], arr, arr[:, -5:], arr[:, -2:]))
 
     def _get_coords_5km(self, scans):
         y = np.arange(self.fscan_len * self.cscan_len) - 2
@@ -144,8 +147,19 @@ class ModisInterpolator():
         x = (np.arange(self.fscan_full_width) - 2) % self.fscan_width
         x[0] = -2
         x[1] = -1
-        x[-2] = 5
-        x[-1] = 6
+        if self.cscan_full_width == 271:
+            x[-2] = 5
+            x[-1] = 6
+        elif self.cscan_full_width == 270:
+            x[-7] = 5
+            x[-6] = 6
+            x[-5] = 7
+            x[-4] = 8
+            x[-3] = 9
+            x[-2] = 10
+            x[-1] = 11
+        else:
+            raise NotImplementedError("Can't interpolate if 5km tiepoints have less than 270 columns.")
         return x, y
 
     def interpolate(self, lon1, lat1, satz1):
@@ -155,21 +169,13 @@ class ModisInterpolator():
         fscan_width = self.fscan_width
         fscan_len = self.fscan_len
 
-        scans = lat1.shape[0] // cscan_len
-        latattrs = lat1.attrs
-        lonattrs = lon1.attrs
-        dims = lat1.dims
-        lat1 = lat1.data
-        lon1 = lon1.data
+        scans = satz1.shape[0] // cscan_len
         satz1 = satz1.data
 
-        lat1 = lat1.reshape((-1, cscan_len, cscan_full_width))
-        lon1 = lon1.reshape((-1, cscan_len, cscan_full_width))
         satz1 = satz1.reshape((-1, cscan_len, cscan_full_width))
 
-        lats_a, lats_b, lats_c, lats_d = get_corners(lat1)
-        lons_a, lons_b, lons_c, lons_d = get_corners(lon1)
         satz_a, satz_b, satz_c, satz_d = get_corners(da.deg2rad(satz1))
+
         c_exp, c_ali = compute_expansion_alignment(satz_a, satz_b, satz_c, satz_d)
 
         x, y = self.get_coords(scans)
@@ -190,24 +196,38 @@ class ModisInterpolator():
         a_track = s_t
         a_scan = (s_s + s_s * (1 - s_s) * c_exp_full + s_t*(1 - s_t) * c_ali_full)
 
-        lats_a = self.expand_tiepoint_array(lats_a, lines, cols)
-        lats_b = self.expand_tiepoint_array(lats_b, lines, cols)
-        lats_c = self.expand_tiepoint_array(lats_c, lines, cols)
-        lats_d = self.expand_tiepoint_array(lats_d, lines, cols)
-        lons_a = self.expand_tiepoint_array(lons_a, lines, cols)
-        lons_b = self.expand_tiepoint_array(lons_b, lines, cols)
-        lons_c = self.expand_tiepoint_array(lons_c, lines, cols)
-        lons_d = self.expand_tiepoint_array(lons_d, lines, cols)
+        res = []
 
-        lats_1 = (1 - a_scan) * lats_a + a_scan * lats_b
-        lats_2 = (1 - a_scan) * lats_d + a_scan * lats_c
-        lats = (1 - a_track) * lats_1 + a_track * lats_2
+        sublat = lat1[::16, ::16]
+        sublon = lon1[::16, ::16]
+        to_cart = abs(sublat).max() > 60 or (sublon.max() - sublon.min()) > 180
 
-        lons_1 = (1 - a_scan) * lons_a + a_scan * lons_b
-        lons_2 = (1 - a_scan) * lons_d + a_scan * lons_c
-        lons = (1 - a_track) * lons_1 + a_track * lons_2
+        if to_cart:
+            datasets = lonlat2xyz(lon1, lat1)
+        else:
+            datasets = [lon1, lat1]
 
-        return xr.DataArray(lons, attrs=lonattrs, dims=dims), xr.DataArray(lats, attrs=latattrs, dims=dims)
+        for data in datasets:
+            data_attrs = data.attrs
+            dims = data.dims
+            data = data.data
+            data = data.reshape((-1, cscan_len, cscan_full_width))
+            data_a, data_b, data_c, data_d = get_corners(data)
+            data_a = self.expand_tiepoint_array(data_a, lines, cols)
+            data_b = self.expand_tiepoint_array(data_b, lines, cols)
+            data_c = self.expand_tiepoint_array(data_c, lines, cols)
+            data_d = self.expand_tiepoint_array(data_d, lines, cols)
+
+            data_1 = (1 - a_scan) * data_a + a_scan * data_b
+            data_2 = (1 - a_scan) * data_d + a_scan * data_c
+            data = (1 - a_track) * data_1 + a_track * data_2
+
+            res.append(xr.DataArray(data, attrs=data_attrs, dims=dims))
+
+        if to_cart:
+            return xyz2lonlat(*res)
+        else:
+            return res
 
 
 def modis_1km_to_250m(lon1, lat1, satz1):
@@ -224,5 +244,22 @@ def modis_1km_to_500m(lon1, lat1, satz1):
 
 def modis_5km_to_1km(lon1, lat1, satz1):
 
-    interp = ModisInterpolator(5000, 1000)
+    interp = ModisInterpolator(5000, 1000, lon1.shape[1])
     return interp.interpolate(lon1, lat1, satz1)
+
+def lonlat2xyz(lons, lats):
+    """Convert lons and lats to cartesian coordinates."""
+    R = 6370997.0
+    x_coords = R * da.cos(da.deg2rad(lats)) * da.cos(da.deg2rad(lons))
+    y_coords = R * da.cos(da.deg2rad(lats)) * da.sin(da.deg2rad(lons))
+    z_coords = R * da.sin(da.deg2rad(lats))
+    return x_coords, y_coords, z_coords
+
+def xyz2lonlat(x__, y__, z__):
+    """Get longitudes from cartesian coordinates.
+    """
+    R = 6370997.0
+    lons = da.rad2deg(da.arccos(x__ / da.sqrt(x__ ** 2 + y__ ** 2))) * da.sign(y__)
+    lats = da.sign(z__) * (90 - da.rad2deg(da.arcsin(da.sqrt(x__ ** 2 + y__ ** 2) / R)))
+
+    return lons, lats
